// BasicShader.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <GL/glew.h>
#include <GL/glut.h>
#include <iostream>
#include <string.h>
#include <fstream>
#include <sstream>
#include <math.h>


GLuint programObject;
GLfloat mouseX=0, mouseY=0;
bool mouseLeftDown=false, mouseRightDown=false;
GLfloat cameraAngleX=0, cameraAngleY=0;
GLfloat cameraDistance=0;
GLUquadricObj *g_Quadric;
GLfloat g_alpha = 0;

const int TEXTURE_WIDTH = 512;
const int TEXTURE_HEIGHT = 512;
GLuint textureId;  

const int WINDOW_WIDTH = 512;
const int WINDOW_HEIGHT = 512;

GLfloat LightPosition[] = {2.0, 6.0, -1.0};//{2.0, 4.0, -1.0};



bool LoadTXTFile(const char *fileName, std::string &textFile)
	{
		std::stringstream strStream;

		std::ifstream file(fileName, std::ifstream::in);
		if(!file.good())
			return false;

		strStream << file.rdbuf();
		textFile = strStream.str();
		file.close();

		return true;
	}

void CalculateAlpha()
{
	GLuint glutTime = glutGet(GLUT_ELAPSED_TIME);

	if(glutTime == 0)// Proti deleni nulou
		glutTime = 1;

	g_alpha = (((float) glutTime / 3000.0f)  - (float) (glutTime/3000))*2.0*3.1415926535;
}


void InitGlut(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);
}

void GLInit()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

	g_Quadric= gluNewQuadric();
	gluQuadricDrawStyle(g_Quadric, GLU_FILL);
	gluQuadricOrientation(g_Quadric, GLU_OUTSIDE);
	gluQuadricNormals(g_Quadric, GLU_SMOOTH);
	gluQuadricTexture(g_Quadric, GLU_TRUE);

	// NEW create a texture object
    glGenTextures(1, &textureId);
	glBindTexture(GL_TEXTURE_2D, textureId);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);//GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST); //GL_LINEAR);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    
    glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, TEXTURE_WIDTH, TEXTURE_HEIGHT, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
	//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TEXTURE_WIDTH, TEXTURE_HEIGHT, 0, GL_RGBA, GL_FLOAT, 0);
    //glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT24, TEXTURE_WIDTH, TEXTURE_HEIGHT, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_INT, 0);
    glBindTexture(GL_TEXTURE_2D, 0);

}

void StoreLightMatrix(GLint unit, GLfloat * projectionMatrix, GLfloat * modelViewMatrix)
{
	GLfloat shiftMatrix[16] = {0.5, 0.0, 0.0, 0.0,
							   0.0, 0.5, 0.0, 0.0,
							   0.0, 0.0, 0.5, 0.0,
							   0.5, 0.5, 0.5, 1.0};

	glActiveTexture(GL_TEXTURE0 + unit);
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(shiftMatrix);
	glMultMatrixf(projectionMatrix);
	glMultMatrixf(modelViewMatrix);
	glMatrixMode(GL_MODELVIEW);
}


void display(void)
{
	GLfloat quadModelViewMatrix[16];
	GLfloat quadProjectionMatrix[16];
	GLfloat teapotModelViewMatrix[16];
	GLfloat teapotProjectionMatrix[16];


	// Vypocti uhel rotace
	CalculateAlpha();

	//Textura hloubky
	glViewport(0, 0, TEXTURE_WIDTH, TEXTURE_HEIGHT);

	//Nastaveni bufferu
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glDrawBuffer(GL_BACK);
	glReadBuffer(GL_BACK);

	//Vykresleni sceny
	glLoadIdentity();
	gluLookAt(LightPosition[0], LightPosition[1], LightPosition[2],
			  0.0, 0.0, 0.0,
			  1.0, 0.0, 0.0);

	glPushMatrix();
	glTranslatef(0.0, -2.0, 0.0);

	glGetFloatv(GL_MODELVIEW_MATRIX, quadModelViewMatrix);
	glGetFloatv(GL_PROJECTION_MATRIX, quadProjectionMatrix);

    glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(-2.0, 0.0, -2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(2.0, 0.0, -2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(2.0, 0.0, 2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(-2.0, 0.0, 2.0);
	glEnd();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.0, sin(g_alpha), 0.0);

	glGetFloatv(GL_MODELVIEW_MATRIX, teapotModelViewMatrix);
	glGetFloatv(GL_PROJECTION_MATRIX, teapotProjectionMatrix);

	glutSolidTeapot(0.5);
	//gluSphere(g_Quadric, 2.0, 30, 20);
	glPopMatrix();

	//Kopirovani hloubky do textury
	glBindTexture(GL_TEXTURE_2D, textureId);
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, TEXTURE_WIDTH, TEXTURE_HEIGHT);


/*


	//Vykresleni textury
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	glRotatef(90.0, 1.0, 0.0, 0.0);
	glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(-2.0, 0.0, -2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(2.0, 0.0, -2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(2.0, 0.0, 2.0);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-2.0, 0.0, 2.0);
	glEnd(); //*/





	// Vymazani bufferu
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Nastaveni kamery
	glLoadIdentity();
	gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	glTranslatef(0, 0, cameraDistance);
	glRotatef(cameraAngleX, 1.0, 0.0, 0.0);
	glRotatef(cameraAngleY, 0.0, 1.0, 0.0);

	// Vykresleni sceny
	glPushMatrix();
	glTranslatef(0.0, -2.0, 0.0);

	StoreLightMatrix(0, quadProjectionMatrix, quadModelViewMatrix);

    glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(-2.0, 0.0, -2.0);

		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(2.0, 0.0, -2.0);

		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(2.0, 0.0, 2.0);

		glNormal3f(0.0, 1.0, 0.0);
		glVertex3f(-2.0, 0.0, 2.0);
	glEnd();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.0, sin(g_alpha), 0.0);

	StoreLightMatrix(0, teapotProjectionMatrix, teapotModelViewMatrix);

	glutSolidTeapot(0.5);
	//gluSphere(g_Quadric, 2.0, 30, 20);
	glPopMatrix(); //*/

	glFlush();
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	        //(Left, Right, Bottom, Top, Near, Far);
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 100.0);
	glMatrixMode(GL_MODELVIEW);
}



void Idle()
{
	glutPostRedisplay();
}

void keyboardSpec(int key, int x, int y)
{
	switch (key){
		case GLUT_KEY_UP:
			cameraAngleX+=1.0;
			glutPostRedisplay();
			break;
		case GLUT_KEY_DOWN:
			cameraAngleX-=1.0;
			glutPostRedisplay();
			break;
		default:
			break;
	}
}
void mouse(int button, int state, int x, int y)
{
    mouseX = x;
    mouseY = y;

    if(button == GLUT_LEFT_BUTTON)
    {
        if(state == GLUT_DOWN)
        {
            mouseLeftDown = true;
        }
        else if(state == GLUT_UP)
            mouseLeftDown = false;
    }

    else if(button == GLUT_RIGHT_BUTTON)
    {
        if(state == GLUT_DOWN)
        {
            mouseRightDown = true;
        }
        else if(state == GLUT_UP)
            mouseRightDown = false;
    }
}

void mouseMotion(int x, int y)
{
    if(mouseLeftDown)
    {
        cameraAngleY += (x - mouseX);
        cameraAngleX += (y - mouseY);
        mouseX = x;
        mouseY = y;
    }
    if(mouseRightDown)
    {
        cameraDistance += (y - mouseY) * 0.2f;
        mouseY = y;
    }
	glutPostRedisplay();
}


void keyboard(unsigned char key, int x, int y)
{if ((int)key == 27) exit(0);}


void GlutCallbacks()
{
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutSpecialFunc(keyboardSpec);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
    glutMotionFunc(mouseMotion);
	glutIdleFunc(Idle);
}

void CheckVersions()
{
	char *version, *extensions, *renderer, *shadingLanguage, *vendor;

	//Dostaneme retezec obsahujici dostupne extenze
    extensions = (char*)glGetString(GL_EXTENSIONS);
	version = (char*)glGetString(GL_VERSION);
	renderer = (char*)glGetString(GL_RENDERER);
	shadingLanguage = (char*)glGetString(GL_SHADING_LANGUAGE_VERSION);
	vendor = (char*)glGetString(GL_VENDOR);

	std::cout << "Vrobce: ";
	std::cout << vendor << "\n";
	std::cout << "Grafick karta: ";
	std::cout << renderer << "\n";
	std::cout << "Verze OpenGL: ";
	std::cout << version << "\n";
	std::cout << "Verze GLSL: ";
	std::cout << shadingLanguage << "\n";
	//std::cout << "Extenze: ";
	//std::cout << extensions;
}

void CreateShader()
{
	GLint status;
	  
	// 1 Vytvoreni objektu shaderu
	GLuint vertexShaderObject;
	GLuint fragmentShaderObject;
	vertexShaderObject= glCreateShader(GL_VERTEX_SHADER);
	fragmentShaderObject= glCreateShader(GL_FRAGMENT_SHADER);
	  
	// 2 Nahrati zdrojoveho kodu ze souboru VertexShader.sh
	std::string vertexShaderTextString;
	std::string fragmentShaderTextString;
	if(LoadTXTFile("VertexShader6.sh", vertexShaderTextString))
		std::cout << "\nVertexShader6.sh loaded\n";
	else std::cout << "\nVertexShader6.sh loading ERROR\n";
	if(LoadTXTFile("FragmentShader6.sh", fragmentShaderTextString))
		std::cout << "FragmentShader6.sh loaded\n";
	else std::cout << "FragmentShader6.sh loading ERROR\n";
	const char * vs= vertexShaderTextString.c_str();
	const char * fs= fragmentShaderTextString.c_str();
	glShaderSource(vertexShaderObject, 1, &vs, NULL);
	glShaderSource(fragmentShaderObject, 1, &fs, NULL);
	  
	// 3 Kompilace shaderu
	glCompileShader(vertexShaderObject);
	glGetShaderiv(vertexShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Vertex shader compilation OK\n";
	else
	{
		std::cout << "\nVertex shader compilation ERROR\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(vertexShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog <<"\n";
	}
	glCompileShader(fragmentShaderObject);
	glGetShaderiv(fragmentShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Fragment shader compilation OK\n";
	else
	{
		std::cout << "\nFragment shader compilation ERROR\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(fragmentShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog <<"\n";
	}
	  
	// 4 Vytvoreni programoveho objektu
	GLuint programObject;
	programObject= glCreateProgram();
	  
	// 5 Pripojeni objektu shaderu k programovemu objektu
	glAttachShader(programObject, vertexShaderObject);
	glAttachShader(programObject, fragmentShaderObject);
	  
	// 6 Slinkovani programu
	glLinkProgram(programObject);
	glGetProgramiv(programObject, GL_LINK_STATUS, &status);
	if(status) std::cout << "Linking OK\n";
	else std::cout <<"Linking ERROR\n";
	  
	// 7 Pouziti programu
	glUseProgram(programObject);

	// zjisteni lokace uniformnich promennych
	GLint LightPositionLoc;
	LightPositionLoc= glGetUniformLocation(programObject, "LightPosition");
	glUniform3fv(LightPositionLoc, 1, LightPosition); //2.0f, 4.0f, -1.0f);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, textureId);
	GLint LightShadowMapLoc;
	LightShadowMapLoc = glGetUniformLocation(programObject, "LightShadowMap");
	glUniform1i(LightShadowMapLoc, 0);

}

int _tmain(int argc, char** argv)
{
	InitGlut(argc, argv);
	GLInit();
	glewInit();

	CheckVersions();
	GlutCallbacks();

	CreateShader();

	glutMainLoop();
	return 0;
}
