// gl_ModelViewProjectionMatrix - aktualni matice prevodu do clipping coordinate system
// gl_ModelViewMatrix - eye coordinates
// gl_ProjectionMatrix
// gl_NormalMatrix
// gl_Vertex
// gl_Normal

varying float LightIntensity;
varying vec4 ShadowCoord;

uniform vec3 LightPosition;

const float SpecularContribution = 0.3;
const float DiffuseContribution = 1.0 - SpecularContribution;


void main()
{
    vec3 ecPosition = vec3(gl_ModelViewMatrix * gl_Vertex);
	vec3 tNorm = normalize(gl_NormalMatrix * gl_Normal);
	vec3 lightVec = normalize(LightPosition - ecPosition);
	vec3 reflectVec = reflect(-lightVec, tNorm);
	vec3 viewVec = normalize(-ecPosition);
	
	float diffuse = max(dot(lightVec,tNorm), 0.0);
	
	float spec = 0.0;
	if (diffuse > 0.0)
	{
	    spec = max(dot(tNorm, normalize(lightVec + viewVec)), 0.0);
	    spec = pow(spec, 16.0);
	}
	
	LightIntensity = DiffuseContribution * diffuse + SpecularContribution * spec;
	
	gl_TexCoord[0] = gl_MultiTexCoord0;
	
	ShadowCoord = gl_TextureMatrix[0] * gl_Vertex;
	
	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}